@@ -54,6 +54,9 @@ TestProxy::~TestProxy()
 
 void TestProxy::onSimpleSignal()
 {
+    m_signalMsg = getProxy().getCurrentlyProcessedMessage();
+    m_signalMemberName = m_signalMsg->getMemberName();
+
     m_gotSimpleSignal = true;
 }
 
@@ -89,28 +92,16 @@ void TestProxy::onPropertiesChanged( const std::string& interfaceName
         m_onPropertiesChangedHandler(interfaceName, changedProperties, invalidatedProperties);
 }
 
-void TestProxy::onInterfacesAdded(const sdbus::ObjectPath& objectPath, const std::map<std::string, std::map<std::string, sdbus::Variant>>& interfacesAndProperties)
-{
-    if (m_onInterfacesAddedHandler)
-        m_onInterfacesAddedHandler(objectPath, interfacesAndProperties);
-}
-
-void TestProxy::onInterfacesRemoved(const sdbus::ObjectPath& objectPath, const std::vector<std::string>& interfaces)
-{
-    if (m_onInterfacesRemovedHandler)
-        m_onInterfacesRemovedHandler(objectPath, interfaces);
-}
-
 void TestProxy::installDoOperationClientSideAsyncReplyHandler(std::function<void(uint32_t res, const sdbus::Error* err)> handler)
 {
     m_DoOperationClientSideAsyncReplyHandler = std::move(handler);
 }
 
-uint32_t TestProxy::doOperationWith500msTimeout(uint32_t param)
+uint32_t TestProxy::doOperationWithTimeout(const std::chrono::microseconds &timeout, uint32_t param)
 {
     using namespace std::chrono_literals;
     uint32_t result;
-    getProxy().callMethod("doOperation").onInterface(sdbus::test::INTERFACE_NAME).withTimeout(500000us).withArguments(param).storeResultsTo(result);
+    getProxy().callMethod("doOperation").onInterface(sdbus::test::INTERFACE_NAME).withTimeout(timeout).withArguments(param).storeResultsTo(result);
     return result;
 }
 
@@ -135,12 +126,12 @@ void TestProxy::doErroneousOperationClientSideAsync()
                                });
 }
 
-void TestProxy::doOperationClientSideAsyncWith500msTimeout(uint32_t param)
+void TestProxy::doOperationClientSideAsyncWithTimeout(const std::chrono::microseconds &timeout, uint32_t param)
 {
     using namespace std::chrono_literals;
     getProxy().callMethodAsync("doOperation")
               .onInterface(sdbus::test::INTERFACE_NAME)
-              .withTimeout(500000us)
+              .withTimeout(timeout)
               .withArguments(param)
               .uponReplyInvoke([this](const sdbus::Error* error, uint32_t returnValue)
                                {
